@@ -82,12 +82,10 @@ namespace libsemigroups {
 
     // Helpers
     template <typename T, typename S>
-    using ReturnTypeIfExists =
-        typename std::enable_if<!std::is_void<T>::value, S>::type;
+    using ReturnTypeIfExists = std::enable_if_t<!std::is_void<T>::value, S>;
 
     template <typename T, typename S>
-    using ReturnTypeIfNotExists =
-        typename std::enable_if<std::is_void<T>::value, S>::type;
+    using ReturnTypeIfNotExists = std::enable_if_t<std::is_void<T>::value, S>;
 
     struct Iterator {};
     struct ConstIterator : Iterator {};
@@ -175,6 +173,7 @@ namespace libsemigroups {
                 typename TOperator = typename TSfinae::LessOrEqualTo>
       auto constexpr operator<=(IteratorBase const& that) const noexcept
           -> ReturnTypeIfNotExists<TOperator, bool> {
+        // Shouldn't this just use < or ==?
         return _wrapped_iter <= that._wrapped_iter;
       }
 
@@ -280,13 +279,13 @@ namespace libsemigroups {
                                                       IteratorBaseAlias>::value)
           : IteratorBaseAlias(it), _state(stt) {}
 
-      IteratorStatefulBase(IteratorStatefulBase const&) = default;
-      IteratorStatefulBase& operator=(IteratorStatefulBase const&) = default;
+      IteratorStatefulBase(IteratorStatefulBase const&);
+      IteratorStatefulBase& operator=(IteratorStatefulBase const&);
 
-      IteratorStatefulBase(IteratorStatefulBase&&) = default;
-      IteratorStatefulBase& operator=(IteratorStatefulBase&&) = default;
+      IteratorStatefulBase(IteratorStatefulBase&&);
+      IteratorStatefulBase& operator=(IteratorStatefulBase&&);
 
-      virtual ~IteratorStatefulBase() = default;
+      virtual ~IteratorStatefulBase();
 
       state_type& get_state() const noexcept {
         return _state;
@@ -303,55 +302,67 @@ namespace libsemigroups {
                 typename TOperator = typename TSfinae::EqualTo>
       auto constexpr operator==(IteratorStatefulBase const& that) const noexcept
           -> ReturnTypeIfExists<TOperator, bool> {
-        return TOperator()(
-            get_state(), this->get_wrapped_iter(), that.get_wrapped_iter());
+        return TOperator()(this->get_state(),
+                           this->get_wrapped_iter(),
+                           that.get_state(),
+                           that.get_wrapped_iter());
       }
 
       template <typename TSfinae   = TIteratorTraits,
                 typename TOperator = typename TSfinae::NotEqualTo>
       auto constexpr operator!=(IteratorStatefulBase const& that) const noexcept
           -> ReturnTypeIfExists<TOperator, bool> {
-        return TOperator()(
-            get_state(), this->get_wrapped_iter(), that.get_wrapped_iter());
+        return TOperator()(this->get_state(),
+                           this->get_wrapped_iter(),
+                           that.get_state(),
+                           that.get_wrapped_iter());
       }
 
       template <typename TSfinae   = TIteratorTraits,
                 typename TOperator = typename TSfinae::Less>
       auto constexpr operator<(IteratorStatefulBase const& that) const noexcept
           -> ReturnTypeIfExists<TOperator, bool> {
-        return TOperator()(
-            get_state(), this->get_wrapped_iter(), that.get_wrapped_iter());
+        return TOperator()(this->get_state(),
+                           this->get_wrapped_iter(),
+                           that.get_state(),
+                           that.get_wrapped_iter());
       }
 
       template <typename TSfinae   = TIteratorTraits,
                 typename TOperator = typename TSfinae::More>
       auto constexpr operator>(IteratorStatefulBase const& that) const noexcept
           -> ReturnTypeIfExists<TOperator, bool> {
-        return TOperator()(
-            get_state(), this->get_wrapped_iter(), that.get_wrapped_iter());
+        return TOperator()(this->get_state(),
+                           this->get_wrapped_iter(),
+                           that.get_state(),
+                           that.get_wrapped_iter());
       }
 
       template <typename TSfinae   = TIteratorTraits,
                 typename TOperator = typename TSfinae::LessOrEqualTo>
       auto constexpr operator<=(IteratorStatefulBase const& that) const noexcept
           -> ReturnTypeIfExists<TOperator, bool> {
-        return TOperator()(
-            get_state(), this->get_wrapped_iter(), that.get_wrapped_iter());
+        return TOperator()(this->get_state(),
+                           this->get_wrapped_iter(),
+                           that.get_state(),
+                           that.get_wrapped_iter());
       }
 
       template <typename TSfinae   = TIteratorTraits,
                 typename TOperator = typename TSfinae::MoreOrEqualTo>
       auto constexpr operator>=(IteratorStatefulBase const& that) const noexcept
           -> ReturnTypeIfExists<TOperator, bool> {
-        return TOperator()(
-            get_state(), this->get_wrapped_iter(), that.get_wrapped_iter());
+        return TOperator()(this->get_state(),
+                           this->get_wrapped_iter(),
+                           that.get_state(),
+                           that.get_wrapped_iter());
       }
 
       template <typename TSfinae   = TIteratorTraits,
                 typename TOperator = typename TSfinae::AddAssign>
       auto operator+=(size_type val) noexcept
           -> ReturnTypeIfExists<TOperator, TSubclass&> {
-        TOperator()(get_state(), this->get_wrapped_iter(), val);
+        TOperator()(this->get_state(), this->get_wrapped_iter(), val);
         return static_cast<TSubclass&>(*this);
       }
 
@@ -359,7 +370,7 @@ namespace libsemigroups {
                 typename TOperator = typename TSfinae::SubtractAssign>
       auto operator-=(size_type val) noexcept
           -> ReturnTypeIfExists<TOperator, TSubclass&> {
-        TOperator()(get_state(), this->get_wrapped_iter(), val);
+        TOperator()(this->get_state(), this->get_wrapped_iter(), val);
         return static_cast<TSubclass&>(*this);
       }
 
@@ -367,14 +378,14 @@ namespace libsemigroups {
       template <typename TSfinae   = TIteratorTraits,
                 typename TOperator = typename TSfinae::PrefixIncrement>
       auto operator++() noexcept -> ReturnTypeIfExists<TOperator, TSubclass&> {
-        TOperator()(get_state(), this->get_wrapped_iter());
+        TOperator()(this->get_state(), this->get_wrapped_iter());
         return static_cast<TSubclass&>(*this);
       }
 
       template <typename TSfinae   = TIteratorTraits,
                 typename TOperator = typename TSfinae::PrefixDecrement>
       auto operator--() noexcept -> ReturnTypeIfExists<TOperator, TSubclass&> {
-        TOperator()(get_state(), this->get_wrapped_iter());
+        TOperator()(this->get_state(), this->get_wrapped_iter());
         return static_cast<TSubclass&>(*this);
       }
 
@@ -382,8 +393,10 @@ namespace libsemigroups {
                 typename TOperator = typename TSfinae::Difference>
       auto constexpr operator-(IteratorStatefulBase const& that) const noexcept
           -> ReturnTypeIfExists<TOperator, difference_type> {
-        return TOperator()(
-            get_state(), this->get_wrapped_iter(), that.get_wrapped_iter());
+        return TOperator()(this->get_state(),
+                           this->get_wrapped_iter(),
+                           that.get_state(),
+                           that.get_wrapped_iter());
       }
 
       template <typename TSfinae   = TIteratorTraits,
@@ -407,6 +420,28 @@ namespace libsemigroups {
       mutable state_type _state;
     };
 
+    template <typename S, typename T>
+    IteratorStatefulBase<S, T>::IteratorStatefulBase(
+        IteratorStatefulBase const&)
+        = default;
+
+    template <typename S, typename T>
+    IteratorStatefulBase<S, T>&
+    IteratorStatefulBase<S, T>::operator=(IteratorStatefulBase const&)
+        = default;
+
+    template <typename S, typename T>
+    IteratorStatefulBase<S, T>::IteratorStatefulBase(IteratorStatefulBase&&)
+        = default;
+
+    template <typename S, typename T>
+    IteratorStatefulBase<S, T>&
+    IteratorStatefulBase<S, T>::operator=(IteratorStatefulBase&&)
+        = default;
+
+    template <typename S, typename T>
+    IteratorStatefulBase<S, T>::~IteratorStatefulBase() = default;
+
     template <typename TSubclass, typename TIteratorTraits>
     class IteratorStatelessBase
         : public IteratorBase<TSubclass, TIteratorTraits> {
@@ -447,17 +482,19 @@ namespace libsemigroups {
 
       template <typename TSfinae   = TIteratorTraits,
                 typename TOperator = typename TSfinae::EqualTo>
-      auto constexpr operator==(IteratorStatelessBase const& that) const
-          noexcept -> typename std::enable_if<
+      auto constexpr
+      operator==(IteratorStatelessBase const& that) const noexcept
+          -> std::enable_if_t<
               std::is_trivially_default_constructible<TOperator>::value,
-              bool>::type {
+              bool> {
         return TOperator()(this->get_wrapped_iter(), that.get_wrapped_iter());
       }
 
       template <typename TSfinae   = TIteratorTraits,
                 typename TOperator = typename TSfinae::NotEqualTo>
-      auto constexpr operator!=(IteratorStatelessBase const& that) const
-          noexcept -> ReturnTypeIfExists<TOperator, bool> {
+      auto constexpr
+      operator!=(IteratorStatelessBase const& that) const noexcept
+          -> ReturnTypeIfExists<TOperator, bool> {
         return TOperator()(this->get_wrapped_iter(), that.get_wrapped_iter());
       }
 
@@ -477,15 +514,17 @@ namespace libsemigroups {
 
       template <typename TSfinae   = TIteratorTraits,
                 typename TOperator = typename TSfinae::LessOrEqualTo>
-      auto constexpr operator<=(IteratorStatelessBase const& that) const
-          noexcept -> ReturnTypeIfExists<TOperator, bool> {
+      auto constexpr
+      operator<=(IteratorStatelessBase const& that) const noexcept
+          -> ReturnTypeIfExists<TOperator, bool> {
         return TOperator()(this->get_wrapped_iter(), that.get_wrapped_iter());
       }
 
       template <typename TSfinae   = TIteratorTraits,
                 typename TOperator = typename TSfinae::MoreOrEqualTo>
-      auto constexpr operator>=(IteratorStatelessBase const& that) const
-          noexcept -> ReturnTypeIfExists<TOperator, bool> {
+      auto constexpr
+      operator>=(IteratorStatelessBase const& that) const noexcept
+          -> ReturnTypeIfExists<TOperator, bool> {
         return TOperator()(this->get_wrapped_iter(), that.get_wrapped_iter());
       }
 
@@ -546,7 +585,7 @@ namespace libsemigroups {
     };
 
     template <typename TIteratorTraits>
-    class ConstIteratorStateful final
+    class ConstIteratorStateful
         : public ConstIterator,
           public IteratorStatefulBase<ConstIteratorStateful<TIteratorTraits>,
                                       TIteratorTraits> {
@@ -611,14 +650,14 @@ namespace libsemigroups {
 
       template <typename TSfinae   = TIteratorTraits,
                 typename TOperator = typename TSfinae::AddressOf>
-      auto operator-> () const noexcept
+      auto operator->() const noexcept
           -> ReturnTypeIfExists<TOperator, const_pointer> {
         return TOperator()(this->get_state(), this->get_wrapped_iter());
       }
 
       template <typename TSfinae   = TIteratorTraits,
                 typename TOperator = typename TSfinae::AddressOf>
-      auto operator-> () const noexcept
+      auto operator->() const noexcept
           -> ReturnTypeIfNotExists<TOperator, const_pointer> {
         return &(*this->get_wrapped_iter());
       }
@@ -696,14 +735,14 @@ namespace libsemigroups {
 
       template <typename TSfinae   = TIteratorTraits,
                 typename TOperator = typename TSfinae::AddressOf>
-      auto operator-> () const noexcept
+      auto operator->() const noexcept
           -> ReturnTypeIfExists<TOperator, const_pointer> {
         return TOperator()(this->get_wrapped_iter());
       }
 
       template <typename TSfinae   = TIteratorTraits,
                 typename TOperator = typename TSfinae::AddressOf>
-      auto operator-> () const noexcept
+      auto operator->() const noexcept
           -> ReturnTypeIfNotExists<TOperator, const_pointer> {
         return &(*this->get_wrapped_iter());
       }
@@ -794,13 +833,13 @@ namespace libsemigroups {
 
       template <typename TSfinae   = TIteratorTraits,
                 typename TOperator = typename TSfinae::AddressOf>
-      auto operator-> () noexcept -> ReturnTypeIfExists<TOperator, pointer> {
+      auto operator->() noexcept -> ReturnTypeIfExists<TOperator, pointer> {
         return TOperator()(this->get_state(), this->get_wrapped_iter());
       }
 
       template <typename TSfinae   = TIteratorTraits,
                 typename TOperator = typename TSfinae::AddressOf>
-      auto operator-> () noexcept -> ReturnTypeIfNotExists<TOperator, pointer> {
+      auto operator->() noexcept -> ReturnTypeIfNotExists<TOperator, pointer> {
         return &(*this->get_wrapped_iter());
       }
 
@@ -886,13 +925,13 @@ namespace libsemigroups {
 
       template <typename TSfinae   = TIteratorTraits,
                 typename TOperator = typename TSfinae::AddressOf>
-      auto operator-> () noexcept -> ReturnTypeIfExists<TOperator, pointer> {
+      auto operator->() noexcept -> ReturnTypeIfExists<TOperator, pointer> {
         return TOperator()(this->get_wrapped_iter());
       }
 
       template <typename TSfinae   = TIteratorTraits,
                 typename TOperator = typename TSfinae::AddressOf>
-      auto operator-> () noexcept -> ReturnTypeIfNotExists<TOperator, pointer> {
+      auto operator->() noexcept -> ReturnTypeIfNotExists<TOperator, pointer> {
         return &(*this->get_wrapped_iter());
       }
 
@@ -902,8 +941,8 @@ namespace libsemigroups {
     };
 
     template <typename TIteratorType,
-              typename = typename std::enable_if<
-                  std::is_base_of<Iterator, TIteratorType>::value>::type>
+              typename = std::enable_if_t<
+                  std::is_base_of<Iterator, TIteratorType>::value>>
     TIteratorType operator+(TIteratorType const& cpy, size_t val) {
       TIteratorType out(cpy);
       out += val;
@@ -911,8 +950,8 @@ namespace libsemigroups {
     }
 
     template <typename TIteratorType,
-              typename = typename std::enable_if<
-                  std::is_base_of<Iterator, TIteratorType>::value>::type>
+              typename = std::enable_if_t<
+                  std::is_base_of<Iterator, TIteratorType>::value>>
     TIteratorType operator-(TIteratorType const& cpy, size_t val) {
       TIteratorType out(cpy);
       out -= val;
@@ -920,15 +959,15 @@ namespace libsemigroups {
     }
 
     template <typename TIteratorType,
-              typename = typename std::enable_if<
-                  std::is_base_of<Iterator, TIteratorType>::value>::type>
+              typename = std::enable_if_t<
+                  std::is_base_of<Iterator, TIteratorType>::value>>
     TIteratorType operator+(size_t val, TIteratorType it) {
       return it + val;
     }
 
     template <typename TIteratorType,
-              typename = typename std::enable_if<
-                  std::is_base_of<Iterator, TIteratorType>::value>::type>
+              typename = std::enable_if_t<
+                  std::is_base_of<Iterator, TIteratorType>::value>>
     TIteratorType operator-(size_t val, TIteratorType it) {
       return it - val;
     }
